# ToDOs:
# child class of Generaldistribution class
# attributes: mean, stdev, n, p
# methods: __init__, calculate_mean, calculate_stdev, replace_stats_with_data(calcuate n, p from read data), plot_bar, pdf, 
# methods: plot_bar_pdf, __add__, __repr__
import math
import matplotlib.pyplot as plt
from .Generaldistribution import Distribution


class Binomial(Distribution):
    def __init__(self, prob=.5, n=10):
        Distribution.__init__(self, mu=0, sigma=1)
        self.p = prob
        self.n = n
    

    def read_data_file(self, filename):
        with open(filename) as file:
            lines = file.readlines()
            datalist = [int(n.strip('\n')) for n in lines]
        self.data = datalist.copy()
        file.close()
        self.p, self.n = self.replace_stats_with_data()


    def calculate_mean(self):
        self.mean = self.p * self.n
        return self.mean

    
    def calculate_stdev(self):
        self.stdev = math.sqrt(self.n * self.p * (1 - self.p))
        return self.stdev
    

    def replace_stats_with_data(self):
        self.n = len(self.data)
        # I think this method of finding p is wrong.
        self.p = 1.0 * sum(self.data) / len(self.data)
        self.mean = self.calculate_mean()
        self.stdev = self.calculate_stdev()

        return self.p, self.n


    def plot_bar(self):
        pass


    def pdf(self, k):
        nCk = math.factorial(self.n) / (math.factorial(self.n - k) * math.factorial(k))
        return nCk * (self.p**k) * ((1 - self.p)**(self.n-k))


    def plot_pdf_bar(self):
        pass


    def __add__(self, other):
        try:
            assert self.p == other.p, 'p values are not equal'
        except AssertionError as Ase:
            raise

        result = Binomial()
        result.p = self.p
        result.n = self.n + other.n
        result.mean = result.calculate_mean()
        result.stdev = result.calculate_stdev()

        return result


    def __repr__(self):
        return "mean: {}, standard deviation: {}, p: {}, n: {}".format(self.mean, self.stdev, self.p, self.n)
    


# ToDos:
# Child class of Distribution class
# attributes: same as parent class
# methods: __init__, read_data_file, calculate_mean, calculate_stdev, plot_histogram, pdf, __add__, __repr__
import math
import matplotlib.pyplot as plt
from .Generaldistribution import Distribution

class Gaussian(Distribution):
    def __init__(self, mu=0, sigma=1):
        Distribution.__init__(self, mu, sigma)


    def read_data_file(self, filename, sample=True):
        with open(filename) as file:
            lines = file.readlines()
            datalist = [float(n.strip('\n')) for n in lines]
        self.data = datalist.copy()
        file.close()
        self.mean = self.calculate_mean()
        self.stdev = self.calculate_stdev(sample)


    def calculate_mean(self):
        try:
            avg = 1.0 * sum(self.data) / len(self.data)
        except ZeroDivisionError as ze:
            return self.mean
        
        self.mean = avg
        return self.mean


    def calculate_stdev(self, sample=True):
        if sample:
            n = len(self.data) - 1
        else:
            n = len(self.data)

        mean = self.calculate_mean()
        sigma = 0

        for x in self.data:
            sigma += (x - mean) ** 2
        sigma = math.sqrt(sigma / n)

        self.stdev = sigma
        return self.stdev
            

    def plot_histogram(self):
        plt.hist(self.data)
        plt.title('Histogram of Data Distribution')
        plt.xlabel('Data')
        plt.ylabel('Count')


    def pdf(self, k):
        return (1.0 / (self.stdev * math.sqrt(2 * math.pi))) * math.exp(-0.5 * ((k - self.mean) / self.stdev)**2)


    def __add__(self, other):
        result = Gaussian()
        result.mean = self.mean + other.mean
        result.stdev = math.sqrt((self.stdev**2) + (other.stdev**2))
        
        return result


    def __repr__(self):
        return "mean: {}, standard deviation: {}".format(self.mean, self.stdev)


# ToDos
# Parent class Gaussian
# attributes: mean, standard_deviation, data
# methods: __init__, read_data_file

class Distribution:
    def __init__(self, mu=0, sigma=1):
        self.mean = mu
        self.stdev = sigma
        self.data = []


    """def read_data_file(self, filename):
        with open(filename) as file:
            datalist = []
            lines = file.readlines()
            for line in lines:
                datalist.append(int(line.strip('\n')))
        file.close()
        self.data = datalist.copy()"""


from setuptools import setup, version

setup(
    name='lhdis',
    version='0.1.1',
    descriptions='Gaussian(Normal) and Binomial Distributions',
    packages=['lhdis'],
    author='Nyi Nyi Linn Htet',
    author_email='nyi20471@gmail.com',
    zip_safe=False
)